/*****************************************************************
 * timesheet.c
 * The Palmpilot Timesheet application.
 * Copyright (C) 1999 Stuart Nicholson
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ******************************************************************/

/*****************************************************************
 * This application represents my first attempt at Palmpilot
 * programming using the GNU Pilot SDK and I begin coding it in an
 * attempt to understand the Pilot GUI and Event model.
 *
 * As such, much of the code below is rather hacky and unclear
 * (particularly those sections that deal with the GUI and moving
 * between Pilot forms).
 ******************************************************************/

/******************************************************************
 * PalmOS includes.
 ******************************************************************/
#include <Common.h>
#include <System/SysAll.h>
#include <System/SysEvtMgr.h>
#include <UI/UIAll.h>

/******************************************************************
 * Application includes.
 ******************************************************************/
#include "callback.h"
#include "resource.h"

#include "Timesheet.h"
#include "Globals.h"

#include "Category.h"
#include "Database.h"

#include "AboutForm.h"
#include "DetailForm.h"
#include "EditCatForm.h"
#include "EditCatEntryForm.h"
#include "MainForm.h"
#include "PrefForm.h"
#include "WeeklyForm.h"
#include "UIUtil.h"

/******************************************************************
 * Application Function Prototypes
 ******************************************************************/

void AppEventLoop(void);
Err StartApplication(void);
Err RomVersionCompatible(DWord requiredVersion, Word launchFlags);
void StopApplication(void);
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags);

/******************************************************************
 * Application global variables
 ******************************************************************/

/* Application copy of the relevant system preferences. */
DateFormatType TSSysDateFormat;
DateFormatType TSSysLongDateFormat;
Byte TSSysWeekStartDay;

/* Application copy of the preferences stored in the database record 0. */
TSAppPrefType TSAppPrefs;

/* Single global entry timer is supported */
TSEntryTimerType TSEntryTimer;

/* The presently open timesheet database. */
DmOpenRef TSDatabase = NULL;

/* True if a record exists for the present day. */
Boolean TSPresentDayRecExists = false;

/* The day record index for the present day (if TSPresentDayRecExists = true),
 * otherwise the record index to insert a day record for the present day.
 */
UInt TSPresentDayRecIdx = TSFirstDayRecIdx;

/******************************************************************
 * Globals relating to the GUI and the presently visible form.
 ******************************************************************/
/* Used to pass information to the Details form. Set this to the database record idx of the entry
 * you're editing, or to 0 if you're attempting to create a *new* entry. */
UInt TSEditEntryIdx = 0;

/* The database record number of the list to edit in the category Edit dialog */
Byte TSEditCatRecIdx = 0;

/* The category entry number of the category beging edited in the category entry Edit dialog, or 0 if you're
 * creating a new category. */
Byte TSEditCatEntryIdx = 0;

/* The present logical day button that is selected on the main form (ie. 0 to 6, not the actual button ID) */
Byte TSSelectedDayButton;

/* The day entry that presently appears at the top of the table in the current form (Main, Weekly or Monthly) */
SWord TSTableTopEntry;

/* The date of the present day being displayed. */
DateType TSPresentDayDate;

/* The adjusted summary date (based on TSPresentDayDate) being displayed. */
DateType TSPresentSummaryDate;

/******************************************************************
 * Run the application event loop.
 ******************************************************************/
void
AppEventLoop(void)
{
  Err error;
  int formID;
  FormPtr formPtr;
  EventType event;

  do
    {
      /* get the next event to handle */
      EvtGetEvent(&event, 50);
      /* let the system have first look at the event */
      if (SysHandleEvent(&event))
        continue;
      if (MenuHandleEvent((void *) 0, &event, &error))
        continue;
      /* handle form loads */
      if (event.eType == frmLoadEvent)
        {
          formID = event.data.frmLoad.formID;
          formPtr = FrmInitForm(formID);
          FrmSetActiveForm(formPtr);
          if (formID == formID_tsMain)
            {
              /* set up the handler for the main form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) MainFormEventHandler);
            }
          else if (formID == formID_tsWeekly)
            {
              /* Set up the handler for the weekly summary form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) WeeklyFormEventHandler);
            }
          else if (formID == formID_tsDetail)
            {
              /* set up the handler for the details form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) DetailFormEventHandler);
            }
          else if (formID == formID_tsEditCat)
            {
              /* set up the handler for the edit form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) EditCatFormEventHandler);
            }
          else if (formID == formID_tsEditCatEntry)
            {
              /* set up the handler for the category edit form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) EditCatEntryFormEventHandler);
            }
          else if (formID == formID_tsPref)
            {
              /* set up the handler for the preferences edit form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) PrefFormEventHandler);
            }
          else if (formID == formID_tsAbout)
            {
              /* set up the handler for the about form */
              FrmSetEventHandler(formPtr, (FormEventHandlerPtr) AboutFormEventHandler);
            }
        }
      FrmDispatchEvent(&event);
    }
  while (event.eType != appStopEvent);
}

/***********************************************************************
 * Checks that Pilot ROM version meets minimum requirement.
 * Returns error code or zero if rom is compatible
 ***********************************************************************/
Err
RomVersionCompatible(DWord requiredVersion, Word launchFlags)
{
  DWord romVersion;

  /* see if we're on in minimum required version of the ROM or later. */
  FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
  if (romVersion < requiredVersion)
    {
      if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
          (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
        {
          FrmAlert(alertID_tsRomIncompat);
          /* Pilot 1.0 will continuously relaunch this app unless we switch to another safe one. */
          if (romVersion < 0x02000000)
            {
              Err err;

              AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
            }
        }
      return sysErrRomIncompatible;
    }
  /* rom version okay */
  return 0;
}

/******************************************************************
 * Start the application, opening databases and selecting initial forms etc.
 ******************************************************************/
Err
StartApplication(void)
{
  Err error;
  SystemPreferencesType sysPrefs;
  DateTimeType now;

  /* Open or create the application database */
  error = OpenDatabase();
  if (error)
    {
      /* Failed to open database */
      return error;
    }
  /* Get a copy of the system's preferences */
  PrefGetPreferences(&sysPrefs);
  /* Save the system preferences we're interested in */
  TSSysDateFormat = sysPrefs.dateFormat;
  TSSysLongDateFormat = sysPrefs.longDateFormat;
  TSSysWeekStartDay = sysPrefs.weekStartDay;
  /* Set the dateTime to today */
  TimSecondsToDateTime(TimGetSeconds(), &now);
  /* Convert DateTime to Date */
  TSPresentDayDate.year = now.year;
  TSPresentDayDate.month = now.month;
  TSPresentDayDate.day = now.day;
  /* Application started okay */
  return 0;
}

/******************************************************************
 * Stop the application, closing any open databases.
 ******************************************************************/
void
StopApplication(void)
{
  LocalID summaryID;
  Int summaryCardNum;
  DmOpenRef summaryDB;

  /* Close the database */
  ClosePreferences();
  DmCloseDatabase(TSDatabase);
  TSDatabase = NULL;
  /* Check if there's an existing summary database */
  summaryDB = DmOpenDatabaseByTypeCreator(TSSummaryDBType, TSAppID, dmModeReadWrite);
  if (summaryDB)
    {
      /* Get locaID, cardID etc. so we can delete database */
      DmOpenDatabaseInfo(summaryDB, &summaryID, NULL, NULL, &summaryCardNum, NULL);
      /* Close and delete existing summary database */
      DmCloseDatabase(summaryDB);
      DmDeleteDatabase(summaryCardNum, summaryID);
    }
}

/******************************************************************
 * Palmpilot main function, entry point for the application.
 ******************************************************************/
DWord
PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
  Word error;
  EventType event;

  if (cmd == sysAppLaunchCmdNormalLaunch)
    {
      /* Check the ROM version is PalmOS 2.0 or better */
      error = RomVersionCompatible(TSRomVersion, launchFlags);
      if (error)
        {
          /* App won't run, wrong ROM version */
          return error;
        }
      else
        {
          /* Run app */
          error = StartApplication();
          if (error)
            {
              /* Failed to start application */
              return error;
            }
          /* Go to the application's initial form */
          FrmGotoForm(formID_tsMain);
          AppEventLoop();
          StopApplication();
        }
    }
  else if (cmd == sysAppLaunchCmdFind)
    {
      FindInDatabase((FindParamsPtr) cmdPBP);
    }
  else if (cmd == sysAppLaunchCmdGoTo)
    {
      /* If the user wants to goto a record listed in the find dialog it sends the following notification.
         Check if new global variables are set up.  This occurs when the system intends for the app to start */
      if (launchFlags & sysAppLaunchFlagNewGlobals)
        {
          /* This occurs when the user wants to see a record in this application's database.
             The app must be started and a view to display the data must be opened */
          /* check the ROM version is PalmOS 2.0 or better */
          error = RomVersionCompatible(TSRomVersion, launchFlags);
          if (error)
            {
              /* App won't run, wrong ROM version */
              return error;
            }
          else
            {
              /* Run app */
              error = StartApplication();
              if (error)
                {
                  /* Failed to start application */
                  return error;
                }
            }
        }
      else
        {
          /* The app was already running, close the existing UI */
          FrmCloseAllForms();
        }
      /* Load the application's main form */
      MemSet(&event, 0, sizeof(EventType));
      event.eType = frmLoadEvent;
      event.data.frmLoad.formID = formID_tsMain;
      EvtAddEventToQueue(&event);
      /* Go to the application's main form */
      event.eType = frmGotoEvent;
      event.data.frmGoto.formID = formID_tsMain;
      event.data.frmGoto.recordNum = ((GoToParamsPtr) cmdPBP)->recordNum;
      EvtAddEventToQueue(&event);
      /* If the app is not already open, run the app */
      if (launchFlags & sysAppLaunchFlagNewGlobals)
        {
          AppEventLoop();
          StopApplication();
        }
    }
  /* App ran normally, or ignored launch code */
  return 0;
}

/* End of file */
